@@ -15,32 +15,50 @@ use Carp;
 
 use Data::Identifier;
 
-use parent qw(Data::Identifier::Interface::Userdata);
-
-our $VERSION = v0.02;
+our $VERSION = v0.03;
 
 
 sub new {
     my ($pkg, $type, $data, %opts) = @_;
+    my $code;
     my $iso;
 
     croak 'Type or data not given' unless defined($type) && defined($data);
 
     if ($type eq 'from') {
-        if ($data =~ /^[12][0-9]{3}(?:-[0-9]{2}(?:-[0-9]{2})?)?Z?$/) {
+        if (eval {$data->isa('SIRTX::Datecode')}) {
+            $data = $data->datecode;
+            $type = 'datecode';
+        } elsif (eval {$data->isa('Data::Identifier') && $data->generator->eq('97b7f241-e1c5-4f02-ae3c-8e31e501e1dc')}) {
+            $data = $data->request;
+            $type = 'iso8601';
+        } elsif (eval {$data->isa('DateTime')}) {
+            if ($data->time_zone->isa('DateTime::TimeZone::Floating')) {
+                $data = $data->ymd;
+                $type = 'iso8601';
+            } else {
+                $data = $data->epoch;
+                $type = 'epoch';
+            }
+        } elsif (eval {$data->can('epoch')}) {
+            $data = $data->epoch;
+            $type = 'epoch';
+        } elsif ($data =~ /^[12][0-9]{3}(?:-[0-9]{2}(?:-[0-9]{2})?)?Z?$/) {
             $type = 'iso8601';
         } elsif ($data eq 'now') {
             $data = time();
             $type = 'epoch';
+        } elsif ($data eq 'null') {
+            $data = 0;
+            $type = 'datecode';
         } else {
             croak 'Invalid data: '.$data;
         }
     }
 
     if ($type eq 'datecode' && $data =~ /^(0|[1-9][0-9]*)$/) {
-        my $code = int $data;
-        return bless(\$code, $pkg);
-    } elsif ($type eq 'epoch' && $data =~ /^(0|[1-9][0-9]*)$/) {
+        $code = int $data;
+    } elsif ($type eq 'epoch' && $data =~ /^-?(0|[1-9][0-9]*)$/) {
         my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(int($data));
         $iso = sprintf('%04u-%02u-%02uZ', $year + 1900, $mon, $mday);
     } elsif ($type eq 'iso8601' && $data =~ /^[12][0-9]{3}(?:-[0-9]{2}(?:-[0-9]{2})?)?Z?$/) {
@@ -51,7 +69,27 @@ sub new {
 
     croak 'Stray options passed' if scalar keys %opts;
 
-    return $pkg->_build_from_iso($iso);
+    $code //= $pkg->_build_from_iso($iso);
+
+    croak 'Invalid datecode value' if $code == 1;
+
+    return bless(\$code, $pkg);
+}
+
+
+sub null {
+    my ($pkg, @opts) = @_;
+
+    croak 'Stray options passed' if scalar @opts;
+
+    return state $null = __PACKAGE__->new(from => 'null');
+}
+
+
+sub now {
+    my ($pkg, @opts) = @_;
+
+    return $pkg->new(from => 'now', @opts);
 }
 
 
@@ -121,11 +159,62 @@ sub iso8601 {
 sub as {
     my ($self, $as, %opts) = @_;
 
+    croak 'No as given' unless defined $as;
+
+    return $self if $as eq 'SIRTX::Datecode';
+
+    croak 'Not supported: This is a null value' if     $self->is_null;
+    croak 'Not supported: This is not in UTC'   unless $self->is_utc;
+
     require Data::Identifier::Generate;
 
     return Data::Identifier::Generate->date($self->iso8601)->as($as, %opts);
 }
 
+
+sub is_utc {
+    my ($self, @opts) = @_;
+
+    croak 'Stray options passed' if scalar @opts;
+
+    return ${$self} & 1;
+}
+
+
+sub is_floating {
+    my ($self, @opts) = @_;
+    return !$self->is_utc(@opts);
+}
+
+
+sub is_null {
+    my ($self, @opts) = @_;
+
+    croak 'Stray options passed' if scalar @opts;
+
+    return ${$self} == 0;
+}
+
+
+sub precision {
+    my ($self, @opts) = @_;
+    my $iso_len = length($self->iso8601 // '');
+
+    croak 'Stray options passed' if scalar @opts;
+
+    if ($iso_len <= 1) {
+        return 'null';
+    } elsif ($iso_len <= 5) {
+        return 'year';
+    } elsif ($iso_len <= 8) {
+        return 'month';
+    } elsif ($iso_len <= 11) {
+        return 'day';
+    }
+
+    croak 'Invalid object state';
+}
+
 # ---- Private helpers ----
 
 sub _build_from_iso {
@@ -158,7 +247,7 @@ sub _build_from_iso {
         $code |= $utc ? 1 : 0;
     }
 
-    return bless(\$code, $pkg);
+    return $code;
 }
 
 1;
@@ -175,7 +264,7 @@ SIRTX::Datecode - module for interacting with SIRTX Datecodes
 
 =head1 VERSION
 
-version v0.02
+version v0.03
 
 =head1 SYNOPSIS
 
@@ -185,8 +274,6 @@ This module provides support to convert between different date formats and SIRTX
 SIRTX datecodes allow to encode dates in the range of years 1582 to 2440 into just 16 bits
 by using a variable precision. They also allow for numeric ordering over their full range.
 
-This package inherits from L<Data::Identifier::Interface::Userdata>.
-
 =head1 METHODS
 
 =head2 new
@@ -217,9 +304,37 @@ The special value C<from> is also supported as C<$type>.
 If C<from> is used an object can be passed that is automatically converted.
 In addition if the value is not reference (object) it is tried to be parsed as per C<iso8601>.
 The special value C<now> can be used to create an object for the current time.
+And the special value C<null> to create an null-object (since v0.03).
+
+Currently C<from> supports at least the following types:
+L<SIRTX::Datecode> (since v0.03),
+some L<Data::Identifier> (since v0.03),
+L<DateTime> (since v0.03),
+and any type that supports a C<epoch()> method (since v0.03).
 
 Currently no options are defined.
 
+=head2 null
+
+    my SIRTX::Datecode $dc = SIRTX::Datecode->null;
+
+(since v0.03)
+
+Returns a null datecode.
+
+B<Note:>
+It is not defined if this method will always return the same object or create a new one each time.
+
+See also: L</is_null>.
+
+=head2 now
+
+    my SIRTX::Datecode $dc = SIRTX::Datecode->now;
+
+(since v0.03)
+
+Returns a new datecode object for the current time.
+
 =head2 datecode
 
     my $datecode = $dc->datecode;
@@ -240,6 +355,43 @@ Returns the datecode as an ISO-8601 date string.
 
 This is a proxy for L<Data::Identifier/as>.
 
+=head2 is_utc
+
+    my $bool = $dc->is_utc;
+
+(since v0.03)
+
+Returns a true-ish value if the datecode is in UTC or a false-ish value if it is floating.
+
+See also: L</is_floating>.
+
+=head2 is_floating
+
+    my $bool = $dc->is_floating;
+
+(since v0.03)
+
+Returns a true-ish value if the datecode is floating or a false-ish value if it is in UTC.
+
+See also: L</is_utc>.
+
+=head2 is_null
+
+    my $bool = $dc->is_null;
+
+(since v0.03)
+
+Returns a true-ish value if the datecode is the null value and false-ish otherwise.
+
+=head2 precision
+
+    my $precision = $dc->precision;
+
+(since v0.03)
+
+Returns the precision of the datecode.
+This is one of: C<null>, C<year>, C<month>, or C<day>.
+
 =head1 AUTHOR
 
 Löwenfelsen UG (haftungsbeschränkt) <support@loewenfelsen.net>
